from onepassword_tools.lib.OnePasswordUtils import OnePasswordUtils
from onepassword_tools.lib.MiscUtils import remove_null_value_keys_in_dict
import uuid
from abc import ABCMeta, abstractmethod


class OnePasswordItem:
    __metaclass__ = ABCMeta

    createCustomUUID: bool = True

    @property
    @abstractmethod
    def item_type(self) -> str:
        pass

    def __init__(self):
        self.opu = OnePasswordUtils()

    def __setitem__(self, key, value):
        setattr(self, key, value)

    def __getitem__(self, item):
        if hasattr(self, item):
            return getattr(self, item)
        else:
            return None

    @abstractmethod
    def _request_object(self):
        """
        Static dictionnary reprensenting the request sent to 1Password to create entry
        :return: dict
        """
        return

    def get(self, item):
        if item is 'notes' and getattr(self, item) is None:
            return ""
        else:
            return self[item]

    def get_uuid_section(self):
        return {
            "title": "",
            "name": "Section_%s" % self.opu.generate_op_section_uuid(),
            "fields": [
                {
                    "t": "UUID",
                    "v": str(uuid.uuid4()),
                    "k": "string",
                    "n": self.opu.generate_op_field_uuid()
                }
            ]
        }

    def get_request_object(self):
        request = self._request_object()

        if self.createCustomUUID:
            request['sections'].append(self.get_uuid_section())

        request = remove_null_value_keys_in_dict(request)

        # ensire notes field present
        if 'notesPlain' not in request.keys():
            request['notesPlain'] = ''

        return request



from onepassword_tools.lib.OnePasswordItem import OnePasswordItem


class OnePasswordServerItem(OnePasswordItem):

    hostname: str = None
    item_type: str = 'Server'
    notes: str = None
    password: str = None
    tags: [] = ['Compte utilisateur']
    url: str = None
    username: str = None

    def __getitem__(self, item):
        if item == 'url':
            return self._get_url()
        else:
            return super().__getitem__(item)

    def __init__(self, **kwargs):
        super().__init__()
        for key, value in kwargs.items():
            if hasattr(self, key):
                self.__setattr__(key, value)

    def _get_url(self):
        if self.username and self.hostname:
            return "ssh://%s@%s" % (self.username, self.hostname)
        elif self.hostname:
            return "ssh://%s" % self.hostname
        else:
            return None

    def _request_object(self):
        """
        Return the dict object with the request sent to 1Password
        :return: dict
        """
        return {
            "notesPlain": self.get('notes'),
            "sections": [
                {
                    "title": "",
                    "name": "Section_%s" % self.opu.generate_op_section_uuid(),
                    "fields": [
                        {
                            "inputTraits": {
                                "keyboard": "URL"
                            },
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('url'),
                            "t": "URL"
                        },
                        {
                            "inputTraits": {
                                "autocapitalization": "none",
                                "autocorrection": "no"
                            },
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('username'),
                            "t": "username"
                        },
                        {
                            "k": "concealed",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('password'),
                            "t": "password"
                        }
                    ]
                },
                {
                    "title": "Admin Console",
                    "name": "Section_%s" % self.opu.generate_op_section_uuid(),
                    "fields": [
                        {
                            "inputTraits": {
                                "keyboard": "URL"
                            },
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "t": "admin console URL"
                        },
                        {
                            "inputTraits": {
                                "autocapitalization": "none",
                                "autocorrection": "no"
                            },
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "t": "admin console username"
                        },
                        {
                            "k": "concealed",
                            "n": self.opu.generate_op_field_uuid(),
                            "t": "console password"
                        }
                    ]
                },
                {
                    "title": "Hosting Provider",
                    "name": "Section_%s" % self.opu.generate_op_section_uuid(),
                    "fields": [
                        {
                            "inputTraits": {
                                "autocapitalization": "Words"
                            },
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "t": "name"
                        },
                        {
                            "inputTraits": {
                                "keyboard": "URL"
                            },
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "t": "website"
                        },
                        {
                            "inputTraits": {
                                "keyboard": "URL"
                            },
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "t": "support URL"
                        },
                        {
                            "inputTraits": {
                                "keyboard": "NamePhonePad"
                            },
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "t": "support phone"
                        }
                    ]
                }
            ]
        }




from onepassword_tools.lib.OnePasswordLoginItem import OnePasswordLoginItem
from onepassword_tools.lib.NewItemCommand import NewItemCommand, new_item_command_options, new_item_command_password
import click


@click.command()
@new_item_command_options
@new_item_command_password
@click.option('--url', help='URL of the website where the account is created', prompt=True, required=True)
@click.option('--username', help='Account username', prompt=True, required=True)
def new_website_account(url, title, username, password, password_length, return_field, vault, account, notes, do_not_ask_credentials):
    """Create a new Login item in 1Password with the given credentials."""
    NewWebsiteAccount(**locals()).run()


class NewWebsiteAccount(NewItemCommand):

    hostname: str = None
    onePasswordItemClass = OnePasswordLoginItem
    titleTemplate = 'ACCOUNT {username} ON {url}'
    url: str = None
    username: str = None




from onepassword_tools.lib.OnePasswordItem import OnePasswordItem


class OnePasswordSSHKeyItem(OnePasswordItem):

    from_user: str = None
    from_host: str = None
    to_host: str = None
    to_host_abbreviated: str = None
    to_ip: str = None
    to_user: str = None
    passphrase: str = None
    public_key: str = None
    private_key: str = None
    item_type: str = 'Server'
    title: str = None
    notes: str = ''
    tags: [] = ['Clef SSH']
    url: str = None
    username: str
    to_port: str = None

    def __getitem__(self, item):
        if item == 'url':
            return self._get_url()
        else:
            return super().__getitem__(item)

    def __init__(self, **kwargs):
        super().__init__()
        for key, value in kwargs.items():
            if hasattr(self, key):
                self.__setattr__(key, value)

    def _get_url(self):
        if self.to_host and self.to_user:
            return "ssh://%s@%s" % (self.to_user, self.to_host)
        else:
            return None

    def _request_object(self):
        """
        Return the dict object with the request sent to 1Password
        :return: dict
        """
        return {
            "notesPlain": self.get('notes'),
            "sections": [
                {
                    "title": "",
                    "name": "Section_%s" % self.opu.generate_op_section_uuid(),
                    "fields": [
                        {
                            "k": "concealed",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('passphrase'),
                            "t": "Passphrase"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('to_host'),
                            "t": "Hostname"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('to_ip'),
                            "t": "IP"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('public_key'),
                            "t": "Public Key"
                        },
                        {
                            "k": "string",
                            "a": {
                                "multiline": "yes"
                            },
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('private_key'),
                            "t": "Private Key"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('url'),
                            "t": "url"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": str(self.get('to_port')),
                            "t": "Port"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('to_host_abbreviated'),
                            "t": "Hostname abbreviated"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('to_user'),
                            "t": "Remote user"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('from_user'),
                            "t": "Local user"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.get('from_host'),
                            "t": "Local host"
                        }
                    ]
                }
            ]
        }
